@@ -1,4 +1,8 @@
 //! RDAP Domain Object Class
+use std::collections::HashSet;
+
+use crate::prelude::{ttl::Ttl0Data, ContentExtensions};
+
 use {
     crate::prelude::{Common, Extension, ObjectCommon},
     buildstructor::Builder,
@@ -145,17 +149,17 @@ impl DsDatum {
         self.events.as_deref().unwrap_or_default()
     }
 
-    /// Returns a u32 if it was given, otherwise None.
+    /// Returns a u32 if it was given; otherwise, None.
     pub fn key_tag(&self) -> Option<u32> {
         self.key_tag.as_ref().and_then(|n| n.as_u32())
     }
 
-    /// Returns a u8 if it was given, otherwise None.
+    /// Returns a u8 if it was given; otherwise, None.
     pub fn algorithm(&self) -> Option<u8> {
         self.algorithm.as_ref().and_then(|n| n.as_u8())
     }
 
-    /// Returns a u8 if it was given, otherwise None.
+    /// Returns a u8 if it was given; otherwise, None.
     pub fn digest_type(&self) -> Option<u8> {
         self.digest_type.as_ref().and_then(|n| n.as_u8())
     }
@@ -221,17 +225,17 @@ impl KeyDatum {
         self.events.as_deref().unwrap_or_default()
     }
 
-    /// Returns a u16 if it was given, otherwise None.
+    /// Returns a u16 if it was given; otherwise, None.
     pub fn flags(&self) -> Option<u16> {
         self.flags.as_ref().and_then(|n| n.as_u16())
     }
 
-    /// Returns a u8 if it was given, otherwise None.
+    /// Returns a u8 if it was given; otherwise, None.
     pub fn protocol(&self) -> Option<u8> {
         self.protocol.as_ref().and_then(|n| n.as_u8())
     }
 
-    /// Returns a u8 if it was given, otherwise None.
+    /// Returns a u8 if it was given; otherwise, None.
     pub fn algorithm(&self) -> Option<u8> {
         self.algorithm.as_ref().and_then(|n| n.as_u8())
     }
@@ -245,7 +249,7 @@ impl KeyDatum {
 /// Represents the DNSSEC information of a domain.
 ///
 /// The following shows how to use the builders to
-/// create a domain with secure DNS informaiton.
+/// create a domain with secure DNS information.
 ///
 /// ```rust
 /// use icann_rdap_common::prelude::*;
@@ -336,17 +340,17 @@ impl SecureDns {
         self.key_data.as_deref().unwrap_or_default()
     }
 
-    /// Returns true if a truish value was given, otherwise false.
+    /// Returns true if a truish value was given; otherwise, false.
     pub fn zone_signed(&self) -> Option<bool> {
         self.zone_signed.as_ref().map(|b| b.into_bool())
     }
 
-    /// Returns true if a truish value was given, otherwise false.
+    /// Returns true if a truish value was given; otherwise, false.
     pub fn delegation_signed(&self) -> Option<bool> {
         self.delegation_signed.as_ref().map(|b| b.into_bool())
     }
 
-    /// Returns max_sig_life as a u64 if it was given, otherwise None.
+    /// Returns max_sig_life as a u64 if it was given; otherwise, None.
     pub fn max_sig_life(&self) -> Option<u64> {
         self.max_sig_life.as_ref().and_then(|n| n.as_u64())
     }
@@ -496,6 +500,9 @@ pub struct Domain {
 
     #[serde(skip_serializing_if = "Option::is_none")]
     pub network: Option<Network>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub ttl0_data: Option<Ttl0Data>,
 }
 
 #[buildstructor::buildstructor]
@@ -529,6 +536,7 @@ impl Domain {
         secure_dns: Option<SecureDns>,
         variants: Vec<Variant>,
         network: Option<Network>,
+        ttl0_data: Option<Ttl0Data>,
         redacted: Option<Vec<crate::response::redacted::Redacted>>,
     ) -> Self {
         Self {
@@ -550,6 +558,7 @@ impl Domain {
             nameservers: to_opt_vec(nameservers),
             public_ids: to_opt_vec(public_ids),
             network,
+            ttl0_data,
         }
     }
 
@@ -585,6 +594,7 @@ impl Domain {
         secure_dns: Option<SecureDns>,
         variants: Vec<Variant>,
         network: Option<Network>,
+        ttl0_data: Option<Ttl0Data>,
         extensions: Vec<Extension>,
         redacted: Option<Vec<crate::response::redacted::Redacted>>,
     ) -> Self {
@@ -607,6 +617,7 @@ impl Domain {
             .and_secure_dns(secure_dns)
             .variants(variants)
             .and_network(network)
+            .and_ttl0_data(ttl0_data)
             .and_redacted(redacted)
             .build();
         domain.common = common;
@@ -641,6 +652,7 @@ impl Domain {
         secure_dns: Option<SecureDns>,
         variants: Vec<Variant>,
         network: Option<Network>,
+        ttl0_data: Option<Ttl0Data>,
     ) -> Self {
         Self {
             common: Common::builder().build(),
@@ -660,6 +672,7 @@ impl Domain {
             nameservers: to_opt_vec(nameservers),
             public_ids: to_opt_vec(public_ids),
             network,
+            ttl0_data,
         }
     }
 
@@ -695,6 +708,7 @@ impl Domain {
         secure_dns: Option<SecureDns>,
         variants: Vec<Variant>,
         network: Option<Network>,
+        ttl0_data: Option<Ttl0Data>,
         extensions: Vec<Extension>,
     ) -> Self {
         let common = Common::level0()
@@ -716,6 +730,7 @@ impl Domain {
             .and_secure_dns(secure_dns)
             .variants(variants)
             .and_network(network)
+            .and_ttl0_data(ttl0_data)
             .build();
         idn.common = common;
         idn
@@ -755,6 +770,11 @@ impl Domain {
     pub fn network(&self) -> Option<&Network> {
         self.network.as_ref()
     }
+
+    /// Getter for the ttl0 data.
+    pub fn ttl0_data(&self) -> Option<&Ttl0Data> {
+        self.ttl0_data.as_ref()
+    }
 }
 
 impl ToResponse for Domain {
@@ -764,14 +784,14 @@ impl ToResponse for Domain {
 }
 
 impl GetSelfLink for Domain {
-    fn get_self_link(&self) -> Option<&Link> {
-        self.object_common.get_self_link()
+    fn self_link(&self) -> Option<&Link> {
+        self.object_common.self_link()
     }
 }
 
 impl SelfLink for Domain {
-    fn set_self_link(mut self, link: Link) -> Self {
-        self.object_common = self.object_common.set_self_link(link);
+    fn with_self_link(mut self, link: Link) -> Self {
+        self.object_common = self.object_common.with_self_link(link);
         self
     }
 }
@@ -798,6 +818,15 @@ impl ObjectCommonFields for Domain {
     }
 }
 
+impl ContentExtensions for Domain {
+    fn content_extensions(&self) -> std::collections::HashSet<super::ExtensionId> {
+        let mut exts = HashSet::new();
+        exts.extend(self.common().content_extensions());
+        exts.extend(self.object_common().content_extensions());
+        exts
+    }
+}
+
 #[cfg(test)]
 #[allow(non_snake_case)]
 mod tests {
@@ -1117,7 +1146,7 @@ mod tests {
             .build();
 
         // WHEN
-        domain = domain.set_self_link(
+        domain = domain.with_self_link(
             Link::builder()
                 .href("http://foo.example")
                 .value("http://foo.example")
