@@ -1,4 +1,8 @@
 //! RDAP IP Network.
+use std::collections::HashSet;
+
+use crate::prelude::ContentExtensions;
+
 use {
     crate::prelude::{Common, Extension, ObjectCommon},
     std::str::FromStr,
@@ -10,10 +14,9 @@ use {
 };
 
 use super::{
-    to_opt_vec,
-    types::{ExtensionId, Link},
-    CommonFields, Entity, Event, GetSelfLink, Notice, Numberish, ObjectCommonFields, Port43,
-    RdapResponseError, Remark, SelfLink, Stringish, ToChild, ToResponse,
+    to_opt_vec, types::Link, CommonFields, Entity, Event, ExtensionId, GetSelfLink, Notice,
+    Numberish, ObjectCommonFields, Port43, RdapResponseError, Remark, SelfLink, Stringish, ToChild,
+    ToResponse,
 };
 
 /// Cidr0 structure from the Cidr0 extension.
@@ -268,7 +271,7 @@ impl Network {
         })
     }
 
-    /// Builds an IP network object for a resopnse.
+    /// Builds an IP network object for a response.
     ///
     /// ```rust
     /// use icann_rdap_common::prelude::*;
@@ -402,14 +405,14 @@ impl ToResponse for Network {
 }
 
 impl GetSelfLink for Network {
-    fn get_self_link(&self) -> Option<&Link> {
-        self.object_common.get_self_link()
+    fn self_link(&self) -> Option<&Link> {
+        self.object_common.self_link()
     }
 }
 
 impl SelfLink for Network {
-    fn set_self_link(mut self, link: Link) -> Self {
-        self.object_common = self.object_common.set_self_link(link);
+    fn with_self_link(mut self, link: Link) -> Self {
+        self.object_common = self.object_common.with_self_link(link);
         self
     }
 }
@@ -436,6 +439,18 @@ impl ObjectCommonFields for Network {
     }
 }
 
+impl ContentExtensions for Network {
+    fn content_extensions(&self) -> std::collections::HashSet<super::ExtensionId> {
+        let mut exts = HashSet::new();
+        exts.extend(self.common().content_extensions());
+        exts.extend(self.object_common().content_extensions());
+        if self.cidr0_cidrs.is_some() {
+            exts.insert(super::ExtensionId::Cidr0);
+        }
+        exts
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use crate::{
@@ -542,7 +557,7 @@ mod tests {
     }
 
     #[test]
-    fn test_big_network_deserializion() {
+    fn test_big_network_deserialization() {
         // GIVEN a big network
         let expected = r#"
         {
